from rest_framework.viewsets import ModelViewSet
from rest_framework import permissions, decorators
from django.contrib.auth import get_user_model
from rest_framework.response import Response

from api import serializers, generics
from web.models import Article


class ArticleViewSet(ModelViewSet):
    queryset = Article.objects.all()
    serializer_class = serializers.ArticleSerializer
    lookup_url_kwarg = 'id'
    method_serializers = {
        'get': serializers.ArticleGetSerializer,
        'patch': serializers.ArticlePartialUpdateSerializer,
    }

    @decorators.action(methods=['GET'], detail=True)
    def hello(self, request, *args, **kwargs):
        return Response({'msg': 'Hello World!'})

    def get_permissions(self):
        if self.request.method in permissions.SAFE_METHODS:
            return []
        return super().get_permissions()

    def get_serializer_class(self):
        method = self.request.method.lower()
        if method in self.method_serializers:
            return self.method_serializers.get(method)

        return self.serializer_class


class UserAPIView(generics.RetrieveCreateAPIView):
    queryset = get_user_model().objects.all()
    method_serializers = {
        'get': serializers.UserGetSerializer,
        'post': serializers.UserCreateSerializer,
    }
